/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Bouncer class


*******************************************************************************
******************************************************************************/

#include "Bouncer.h"

#define MIN_X ( -32 << 12 )
#define MIN_Y ( -18 << 12 )

#define BOUNCER_MAX_HP (264 * 4)

//------------------------------------------------------------------------------
//	Bouncer Enemy
//------------------------------------------------------------------------------
Bouncer::Bouncer()
{
	x = 600 <<12;
	y = 96 << 12;
	Frame = 0;
	Active = false;	
	Alive = true;
	DestroyedCounter = 60 * 5;
	Hp = 38 * 4; 
	DrawMode = EN_NORMAL;
	FlipMode = GL_FLIP_NONE;
}

Bouncer::~Bouncer()
{
}


void Bouncer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
}

void Bouncer::Update()
{
	const s32 Delta = BRAD_PI/64;
	static s32 Rotation = 0;
	
	if( Active )
	{
	
		Rotation += Delta;
		x += Dx;
		Dy = -abs(sinLerp( Rotation )) * Radius;
		y = Sy + Dy;
		
		if( (Rotation & (BRAD_PI-1)) == 0 )
		{
			Radius = 20 + qran_range(0,100);
			
		}
		
		if(Counter > 64*3)
		{
			if( (Rotation & (BRAD_PI-1)) == 0 )
			{
				if (qran_range(0,255)> 127) Dx = -Dx;
			}
		
			if( x < 0) Dx = -Dx;
			if( x > (256 << 12) ) Dx = -Dx;
		}
		
		Counter++;
		// animate
		FlipMode = GL_FLIP_NONE;
		s32 Ani = abs( (Rotation & (BRAD_PI-1)));
		if(Ani == 0) Frame = 0;
		
		if( (Ani > 1024) && (Ani < 2048)) Frame = 1;
		
		if( (Ani > 2048) && (Ani < 4096)) Frame = 2;
		
		if( (Ani > 4096)) Frame = 3;
		
		//if( ( Counter & 3 ) == 0 )
		//{
		//	Frame = (Frame + 1) & 3;
		//}
	}
	
	if( Active ) Targets::Instance()->AddCoord( x, y );
}
	

void Bouncer::Draw()
{
	glColor( RGB15(31,31,31) );
	if( Active )
	{
		if( (DrawMode == EN_WHITE) )
		{
			DrawMode = EN_NORMAL;
			glColor( RGB15(31,0,0) );
			glSprite( (x >> 12) - 32, (y >> 12) - 32, 
					  FlipMode, &Images[Frame] 
					);
			glColor( RGB15(31,31,31) );

		}
		else
		{
			glSprite( (x >> 12) - 32, (y >> 12) - 32, 
					  FlipMode, &Images[Frame] 
					);
		}
		
	}

	// draw energy
	int xx = (x >> 12) - 32;
	int yy = (y >> 12) - 45;
	
	int Length = 64;
	s32 Gauge = (Hp << 12) / BOUNCER_MAX_HP;
	int GaugeLength = ( UTIL::Lerpf32( 0, Length << 12, Gauge ) );
	glBoxFilledGradient( xx, yy,
						xx + (GaugeLength >> 12), 
						yy+ 4,
						RGB15(31,0,0),
						RGB15(31,0,0),
						RGB15(0,31,0),
						RGB15(0,0,31) );
	
	glBox(xx, yy, xx + (Length), yy + 4, RGB15(Counter, Counter/2, 31-Counter) );
	
}


bool Bouncer::Collide( StarFighter &Star )
{
	
	if( Active )
	{
		int rad2 = 32 * 32;
		int cx = (x >> 12) - (Star.x >> 12);
		int cy = (y >> 12) - (Star.y >> 12);
		if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
		{
			if( Star.IsBursting() )
			{
				Vector2df32 p;
				p.x = Star.x;
				p.y = Star.y;
				Particle::Instance()->Spawn( p, 4 );
				Star.IncreaseGauge(10);
				Hp -= 4;
				if( (Counter & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
				if( Hp < 0 )
				{
					Active = false;
					Alive = false;
					Star.Score += 21130;
					Sound::Instance()->PlaySFX(E_SFX_BIGEXP);
					// Add some nice explosions
					Explosion::Instance()->Spawn( x >> 12,
												  y >> 12,
												   0,
												   0,
												   Explode::MEDIUM_2
												 );
						
					
					for( int j = 0; j < 8; j++ )
					{	
						s32 dx = cosLerp(BRAD_PI/8 + j * BRAD_PI / 4);
						s32 dy = sinLerp(BRAD_PI/8 + j * BRAD_PI / 4);
						Explosion::Instance()->Spawn( x >> 12,
												  y >> 12,
												   dx,
												   dy,
												   Explode::MEDIUM_1
												 );
						
					}
					
					Explosion::Instance()->MegaSpawn(x >> 12,y >> 12,80,Explode::TINY,Explode::MEDIUM_5, 60*4 );
	
					x = 600 << 12;
					y = 96 << 12;
	
				}
				if( DrawMode == EN_NORMAL )
				{
					if( (Counter & 1) ) DrawMode = EN_WHITE;
				}
			}
			else
			{
				// kill star
				if( !Star.IsInvincible() ) Star.Kill();
			}
			return true;
		}
	}
	
	
	// missiles if after star enemy is still active
	if( Active )
	{
		int rad2 = 32 * 32;
		
		if( x < ( 256 << 12 ) )
		{
		
			// Missiles
			for( int i = 0; i < MAX_MISSILES; i++)
			{
				HomingMissile Missile= Star.GetMissile(i);
				
				if( Missile.IsActive() )
				{
					int cx = (x >> 12) - (Missile.GetX() >> 12);
					int cy = (y >> 12) - (Missile.GetY() >> 12);
					if( ( (cx * cx) + (cy * cy) ) < (rad2) )
					{
						Vector2df32 p;
						p.x = Missile.GetX();
						p.y = Missile.GetY();
						Particle::Instance()->Spawn( p, 1 );
						if( (Counter & 1) ) DrawMode = EN_WHITE;
						Star.DestroyMissile(i);
						Hp -= 1;
						if( (Counter & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
						if( Hp < 0 )
						{
							Active = false;
							Alive = false;
							Star.Score += 11130;
							Sound::Instance()->PlaySFX(E_SFX_BIGEXP);
							// Add some nice explosions
							Explosion::Instance()->Spawn( x >> 12,
														  y >> 12,
														   0,
														   0,
														   Explode::MEDIUM_2
														 );
							
							for( int j = 0; j < 8; j++ )
							{	
								s32 dx = cosLerp(BRAD_PI/8 + j * BRAD_PI / 4);
								s32 dy = sinLerp(BRAD_PI/8 + j * BRAD_PI / 4);
								Explosion::Instance()->Spawn( x >> 12,
														  y >> 12,
														   dx,
														   dy,
														   Explode::MEDIUM_1
														 );
								
							}
							
							Explosion::Instance()->MegaSpawn(x >> 12,y >> 12,80,Explode::TINY,Explode::MEDIUM_5, 60*4 );

							x = 600 << 12;
							y = 96 << 12;
			
						}
					}
			
				}
			}
		
		}   // iter x < 

	}   // iter active

	return false;
}



void Bouncer::Spawn()
{
	
	
	x = 300 << 12;
	y = 0;
	Dx = -floattof32(0.75);
	Dy = 0;
	Sy = (191-32) << 12;
	Radius = 100;
		
	Active = true;
	Alive = true;	
	DestroyedCounter = 60 * 5;
	Frame = 0;
	Counter = 0;
	Hp = 264 * 4;

	
}

