/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	StarFighter class


*******************************************************************************
******************************************************************************/

#include "StarFighter.h"

#define MIN_X ( 0 )
#define MIN_Y ( 0 )

#define MAX_X ( SCREEN_WIDTH << 12 )
#define MAX_Y ( ( SCREEN_HEIGHT-8 ) << 12 )

#define  MAX_MISSILE_DELAY 16

static const s32 X_FRICTION = floattof32(0.006);
static const s32 Y_FRICTION = floattof32(0.008);
static const s32 GRAVITY = floattof32(0.012);
static const s32 ACCEL = floattof32(0.02);


//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
StarFighter::StarFighter()
{
	Score = 0;
	Lives = 2;
	IsAlive = true;
	NewLifeCounter = 64;
	StartFrame = 6;
	Frame = 0;
	Counter = 0;
	BurstCounter = 0;
	Direction = 1;
	x = -20 << 12;
	y = ( SCREEN_HEIGHT/2 ) << 12;
	Dx = 0;
	Dy = 0;
	FlipMode = GL_FLIP_NONE;
	LaserReady = false;
	LaserGauge = LASER_GAUGE_FULL;
	
	NewLife();
	
}

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
StarFighter::~StarFighter()
{

}

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
bool StarFighter::Update( int Key, int KeyD )
{
	
	Counter++;
	
	if( x > (300<<12) )
	{
		return true;
	}
	
	if( StateCounter > 0 )
	{
		StateCounter--;
		if( StateCounter == 0 )
		{
			State &= ~E_INVINCIBLE;
		}
	}
	
	// Manual Control
	if( !(State & E_LOCKED) )
	{
		// process input
		if( Key & KEY_RIGHT )
		{
			Dx += ( ( ACCEL >> 4 ) * X_FRICTION );
			Direction = 1;
		}
		
		if( Key & KEY_LEFT )
		{
			Dx -= ( ( ACCEL >> 4 ) * X_FRICTION );
			Direction = -1;
		}
		
		if( Key & KEY_DOWN )
		{
			Dy += ( ( ACCEL >> 4 ) * Y_FRICTION );
		}

		if( (Key & KEY_UP) )
		{
			Dy -= ( ( ACCEL >> 3 ) * Y_FRICTION );
		}
		
			
		if( (KeyD & KEY_B) )
		{
			if( BurstCounter <= 0 )
			{
				BurstCounter = 32;
				Sound::Instance()->PlaySFX(E_SFX_BURST);
			}
			
		}
		
		if( (KeyD & KEY_A) )
		{
			//if( LaserCounter < 0 )
			if( (NumMissiles == 0) && (LaserGauge >= LASER_GAUGE_FULL) )
			{
				LaserReady = true;
				LaserGauge = 0;
			}
		}
		
		if( LaserGauge < LASER_GAUGE_FULL )
		{
			LaserGauge++;
		}
		else
		{
			if( (Counter & 7) == 0 ) Sound::Instance()->PlaySFX(E_SFX_LASER_FULL);
		}				
		
		if( NumMissiles == MAX_MISSILES ) 
		{
			LaserReady = false;
			
		}
		
		//if( NumMissiles > 0 ) 
		//{
		//	State |= E_INVINCIBLE;
		//}
		
		// Missiles
		if( LaserReady ) 
		{
			MissileSpawnDelay--;
			if( MissileSpawnDelay < 0 )
			{
				Sound::Instance()->PlaySFX(E_SFX_LASER);
				MissileSpawnDelay = MAX_MISSILE_DELAY;
				SpawnMissile();
				Explosion::Instance()->Spawn( (x >> 12) - 16,
											  (y >> 12),
										      0,
										      0,
									          Explode::MEDIUM_3);
	
			}
		}
			
		FlipMode = GL_FLIP_NONE;
		
		if( BurstCounter <= 0 )
		{
		
			Dx -= ( Dx * X_FRICTION ) >> 12;
			Dy += GRAVITY;
			Dy -= ( Dy * Y_FRICTION ) >> 12;

			x += Dx;
			y += Dy;

			// animate
			StartFrame = 6;
			if( ( Counter & 3 ) == 0 )
			{
				if( Direction > 0 )
					Frame = (Frame + 1) & 7;
				else
					Frame = (Frame - 1) & 7;
			}
			
			Bounce();
		}
		else
		{
			BurstCounter--;
			Dx = 0;
			Dy = 0;
			
			if( Direction < 0 )
			{
				x -= 3 << 12;
				FlipMode = GL_FLIP_H;
			}
			else
			{
				x += 3 << 12;
			}
			
			Limit();
			
			// animate
			StartFrame = 0;
			Frame = (Frame + 1);
			if( Frame > 5) Frame = 0;
		
		}
	}
	else		// Auto mode
	{
		if( Mode == E_ENTER )
		{
			if( x < (60 << 12) ) 
			{
				FlipMode = GL_FLIP_NONE;
				BurstCounter = 0;
				Direction = 1;
				x += 2 << 12 ;
			}
			else
			{
				State &= ~E_LOCKED;
			}
		}
		else  //exit
		{
			if( x < (350 << 12) ) 
			{
				FlipMode = GL_FLIP_NONE;
				BurstCounter = 0;
				Direction = 1;
				x += 2 << 12 ;
			}
		}
		
		
		// animate
		StartFrame = 0;
		Frame = (Frame + 1);
		if( Frame > 5) Frame = 0;
	
	}
	
	return false;
	
	
}



//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
void StarFighter::UpdateMissiles()
{
	
	NumMissiles = 0;
	for( int i = 0; i < MAX_MISSILES; i++)
	{
		if( Missiles[i].IsActive() )
		{
			NumMissiles++;
			Missiles[i].Update();
		}
	}
	
}


//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
void StarFighter::Draw( int Palette, glImage* const Sprites ) const
{
	
	// flicker
	if( IsInvincible() && (((Counter+1) & 1)) && !IsLocked() ) return;
	
	// Center the sprite
	// From now on all the sprites are gonna be centerbased
	// Meaning X and Y are the centers
	glColor( RGB15(31,31,31) );
	if( (Counter & 1) && (BurstCounter > 0) )
	{
		if(IsInvincible())
			glColor( RGB15(31,31,31) );
		else
			glColor( RGB15(0,0,31) );
		glColor( RGB15(qran_range(0, 31), qran_range(0, 31), Counter&31) );
	}
	
	int offx = Sprites[Frame+StartFrame].width/2;
	int offy = Sprites[Frame+StartFrame].height/2;
	glSprite( (x >> 12)-offx, (y >> 12)-offy, FlipMode, &Sprites[Frame+StartFrame] );
	
	
	
}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::DrawMissiles( int Palette, glImage* const Sprites, glImage* const Sprites2 )
{

	
	// Draw the flares if lasergauge is full
	if( !IsInvincible() )
	{
		if( ( LaserGauge >= LASER_GAUGE_FULL ) && !IsBursting() )
		{	
			u8 r = abs(sinLerp( Counter * 5) * 31);
			u8 g = abs(sinLerp( Counter * 6) * 31);
			u8 b = abs(sinLerp( Counter * 7) * 31);
			glColor( RGB15(31,g,b) );
			glPolyFmt( POLY_ALPHA(15) | POLY_CULL_NONE | POLY_ID( GlobalManager::Instance()->GetFreePolyID() ) );
			glSpriteRotateScaleXY( x >> 12, y >> 12, 0, sinLerp(Counter*210) * 1, sinLerp(Counter*210) * 1,  GL_FLIP_NONE, &Sprites2[2] );
			glColor( RGB15(r,31,b) );
			glPolyFmt( POLY_ALPHA(15) | POLY_CULL_NONE | POLY_ID( GlobalManager::Instance()->GetFreePolyID() ) );
			glSpriteRotateScaleXY( x >> 12, y >> 12, 0, cosLerp(Counter*210) * 1, cosLerp(Counter*210) * 1,  GL_FLIP_NONE, &Sprites2[2] );
		}
	}
	
	// Missiles
	for( int i = 0; i < MAX_MISSILES; i++)
	{
		if( Missiles[i].IsActive() )
		{
			
			//if( !Missiles[i].GetIsDone() )
			//{
				int df = Missiles[i].GetDrawFrame();
				glSpriteRotate( (Missiles[i].GetX() >> 12),
								(Missiles[i].GetY() >> 12),
								Missiles[i].GetAngle(),
								GL_FLIP_NONE, 
								&Sprites[df] );
			//}
			
			Missiles[i].DrawLinks( RGB15(0,31,0), Sprites2 );
		}
	}
	
	
	glPolyFmt( POLY_ALPHA(31) | POLY_CULL_NONE | POLY_ID( GlobalManager::Instance()->GetFreePolyID() ) );
			
	
}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::DrawLaserGauge( glImage* const Sprites, glImage* const Sprites2 ) const
{
	static int anim = 0;
	int x1 = 0;
	int y1 = 185;
	int Length = 128;
	s32 Gauge = (LaserGauge << 12) / LASER_GAUGE_FULL;
	int GaugeLength = ( UTIL::Lerpf32( 0, Length << 12, Gauge ) );
	
	// animate
	if( ( Counter & 3 ) == 0 )
	{
		anim++;
	}
	
	
	glColor( RGB15(31,31,31) );
	int f = anim % 3;
	glSpriteStretchHorizontal( x1, y1-10, Length + 28, &Sprites2[f] );
	
	
	int Color = RGB15(31,31,31);
	
	if( LaserGauge >= LASER_GAUGE_FULL ) 
	{
		f = anim & 3;
		Color = RGB15(31,31,31);
	}
	else
	{
		f = 0;
		Color = RGB15(24,0,31);
	}
	
	glColor( Color );
	glSpriteStretchHorizontal( x1+11, y1 - 3, 8 + (GaugeLength >> 12), &Sprites[f] );
	
	glColor( RGB15(31,31,31) );
	
	
}
		

//--------------------------------------
//	
//--------------------------------------
void StarFighter::NewLife()
{
	State = 0;
	State = E_LOCKED | E_INVINCIBLE;
	Mode = E_ENTER;
	
	StateCounter = 60 * 4;
	
	x = -30 << 12;
	y = 96 << 12;
	Dx = 0;
	Dy = 0;
	
	Active = true;
	IsAlive = true;
	
	MissileSpawnDelay = 0;
	MissileFrameOffset = 0;
	MissileRotation = 1;
	NumMissiles = 0;
	
	LaserReady = false;
	LaserGauge = LASER_GAUGE_FULL;
	
}
		
//--------------------------------------
//	
//--------------------------------------
void StarFighter::Kill()
{
	Lives--;
	NewLifeCounter = 60 * 2;
	IsAlive = false;
	LaserGauge = 0;
	Sound::Instance()->PlaySFX(E_SFX_PLAYERMISS);
	
	// Add some nice explosions
	Explosion::Instance()->Spawn( x >> 12,
								  y >> 12,
								   0,
								   0,
								   Explode::MEDIUM_3,
								   Explode::SMALL_1
								 );
		


	x = -30 << 12;
	y = 96 << 12;
	State = E_LOCKED | E_INVINCIBLE;
	
}

//--------------------------------------
//	Private Functions
//--------------------------------------
void StarFighter::Bounce()
{

	if( (x < MIN_X) )
	{
		Dx = -Dx;
		x = MIN_X;
	}
	if( (x > MAX_X) )
	{
		Dx = -Dx;
		x = MAX_X;
	}
	if( (y < MIN_Y) )
	{
		Dy = -Dy;
		y = MIN_Y;
	}
	if( (y > MAX_Y) )  
	{
		Dy = -Dy;
		y = MAX_Y;
	}

}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::Limit()
{

	if( (x < MIN_X) )
	{
		x = MIN_X;
	}
	if( (x > MAX_X) )
	{
		x = MAX_X;
	}
	if( (y < MIN_Y) )
	{
		y = MIN_Y;
	}
	if( (y > MAX_Y) )  
	{
		y = MAX_Y;
	}

}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::SetTargets()
{

	int NumTargets = Targets::Instance()->GetNumTargets();
		
	if( NumTargets == 0)
	{
		for( int j = 0; j < MAX_MISSILES; j++ )
		{
			Missiles[j].SetTarget( 128 << 24, y );
		}
	}
	else
	{
		int StartTarget = 0;
		for( int j = 0; j < MAX_MISSILES; j++ )
		{
			TargetEntity T = Targets::Instance()->GetTarget(StartTarget);
			Missiles[j].SetTarget( T.GetX(), T.GetY() );
			StartTarget++;
			if( StartTarget >= ( NumTargets - 1 ) ) StartTarget = 0;
			
		}  //next j
	}  // end if

}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::KillArmament()
{
	
	for( int i = 0; i < MAX_MISSILES; i++)
	{
		Missiles[i].Kill();
	}  	
}

//--------------------------------------
//	
//--------------------------------------
bool StarFighter::SpawnMissile()
{

	//bool Spawned = false;
	
	if( NumMissiles > MAX_MISSILES - 2 ) return false;
	
	for( int j = 0; j < 2; j++)
	{
		for( int i = 0; i < MAX_MISSILES; i++)
		{
			if( !Missiles[i].IsActive() )
			{
				Missiles[i].Init( x - (4 << 12), y, MissileFrameOffset, MissileRotation );
				Missiles[i].SetEnergy(8);
				MissileFrameOffset++;
				if( MissileFrameOffset > 2 ) MissileFrameOffset = 0;
				
				MissileRotation = -MissileRotation;
				//Spawned = true;
				break;
			}
		}
	}
	
	return true;
	
	//if( Spawned ) Sound::Instance()->PlaySFX( E_SFX_MISSILE );

}

	



